@@ -1,7 +1,7 @@
 __author__ = 'Altertech, http://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2018-2020 Altertech Group'
 __license__ = 'Apache License 2.0'
-__version__ = '2.0.42'
+__version__ = '2.0.43'
 
 from flask import request, jsonify, Response, abort, send_file, make_response
 from flask_restx import Api, Resource, reqparse
@@ -46,22 +46,25 @@ from pyaltt2.network import netacl_match
 
 success = {'ok': True}
 
-authenticators = []
+filters = []
 """
-custom authenticator is a method defined as:
+custom filter is a method defined as:
 
     def myauth(ip, f, *args, **kwargs):
         # ip - remote IP address
         # f - requested API function
         # *args, **kwargs - function arguments
 
-custom authenticator return values:
+custom filter return values:
 
-* True: API request can be passed
-* None or False: ignored, go to next authenticator
+* True: API request can be passed to the API function
+* None or False: ignored, go to next filter
+* any other value: returned to user as-is, chain is stopped, API function is
+  not called
 
-in case of unsuccessful authentication attempt, custom authenticator should
-call abort(status) or raise an exception
+in case if filter is used as an authenticator and unsuccessful authentication
+attempt has been performed, the filter should call abort(status) or raise an
+exception
 """
 
 
@@ -149,7 +152,7 @@ def public_method(f):
     return do
 
 
-def master_authenticator(ip, f, *args, **kw):
+def master_filter(ip, f, *args, **kw):
     logger.debug(f'API admin call {ip} method: {f.__qualname__}, args: {args}, '
                  f'kwargs: {kw}')
     key = request.headers.get('X-Auth-Key', kw.get('k'))
@@ -168,18 +171,21 @@ def master_authenticator(ip, f, *args, **kw):
     return True
 
 
-def authenticated_method(f):
+def filtered_method(f):
 
     @wraps(f)
     def do(*args, **kwargs):
         ip = get_real_ip()
         payload = request.json if request.json else {}
         kw = {**kwargs, **payload}
-        for c in authenticators:
-            if c(ip, f, *args, **kw):
+        for c in filters:
+            result = c(ip, f, *args, **kw)
+            if result is True:
                 break
+            elif result is not False and result is not None:
+                return result
         else:
-            if not master_authenticator(ip, f, *args, **kw):
+            if not master_filter(ip, f, *args, **kw):
                 abort(403)
         return f(*args, **kw)
 
@@ -587,14 +593,14 @@ def _process_addr(a):
         return (None, a if isinstance(a, str) else None)
 
 
-@authenticated_method
+@filtered_method
 def test(**kwargs):
     result = success.copy()
     result.update({'version': product.version, 'build': product.build})
     return jsonify(result)
 
 
-@authenticated_method
+@filtered_method
 def r_core_cmd(cmd=None):
     if cmd == 'reset-addr-limits':
         reset_addr_limits()
@@ -607,7 +613,7 @@ def r_core_cmd(cmd=None):
     return _response_empty()
 
 
-@authenticated_method
+@filtered_method
 def r_plugin_list():
     result = []
     for k, v in plugin_list().items():
@@ -620,12 +626,12 @@ def r_plugin_list():
     return jsonify(sorted(result, key=lambda k: k['plugin_name']))
 
 
-@authenticated_method
+@filtered_method
 def r_addr_list():
     return jsonify(addr_list())
 
 
-@authenticated_method
+@filtered_method
 def r_addr_get(a):
     addr_id, addr = _process_addr(a)
     try:
@@ -634,7 +640,7 @@ def r_addr_get(a):
         return _response_not_found(f'addr {a} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_addr_cmd(a, cmd, to=None):
     addr_id, addr = _process_addr(a)
     try:
@@ -650,7 +656,7 @@ def r_addr_cmd(a, cmd, to=None):
         return _response_not_found(f'addr {a} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_addr_create():
     result = addr_get(addr_create())
     return _response_created(
@@ -658,7 +664,7 @@ def r_addr_create():
         'addr')
 
 
-@authenticated_method
+@filtered_method
 def r_addr_modify(a, active=None, lim_c=None, lim_s=None):
     addr_id, addr = _process_addr(a)
     try:
@@ -676,7 +682,7 @@ def r_addr_modify(a, active=None, lim_c=None, lim_s=None):
         return _response_not_found(f'addr {a} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_addr_delete(a):
     addr_id, addr = _process_addr(a)
     try:
@@ -697,7 +703,7 @@ def _get_object_verify_addr(obj_id, a, getfunc, **kwargs):
         raise LookupError
 
 
-@authenticated_method
+@filtered_method
 def r_endpoint_list(a):
     addr_id, addr = _process_addr(a)
     try:
@@ -707,7 +713,7 @@ def r_endpoint_list(a):
         return _response_not_found(f'addr {a} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_endpoint_get(a, ep):
     try:
         endpoint = _get_object_verify_addr(ep, a, endpoint_get)
@@ -716,7 +722,7 @@ def r_endpoint_get(a, ep):
         return _response_not_found(f'endpoint {a}/{ep} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_endpoint_cmd(a, ep, cmd, **kwargs):
     try:
         endpoint = _get_object_verify_addr(ep, a, endpoint_get)
@@ -743,7 +749,7 @@ def r_endpoint_cmd(a, ep, cmd, **kwargs):
         return _response_not_found(f'endpoint {a}/{ep} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_endpoint_create(a, plugin_name, **kwargs):
     addr_id, addr = _process_addr(a)
     try:
@@ -766,7 +772,7 @@ def r_endpoint_create(a, plugin_name, **kwargs):
         f'addr/{a}/endpoint')
 
 
-@authenticated_method
+@filtered_method
 def r_endpoint_modify(a, ep, **kwargs):
     try:
         endpoint = _get_object_verify_addr(ep, a, endpoint_get)
@@ -788,7 +794,7 @@ def r_endpoint_modify(a, ep, **kwargs):
         'roboger.endpoint') else _response_empty()
 
 
-@authenticated_method
+@filtered_method
 def r_endpoint_delete(a, ep):
     try:
         endpoint = _get_object_verify_addr(ep, a, endpoint_get)
@@ -798,7 +804,7 @@ def r_endpoint_delete(a, ep):
         return _response_not_found(f'endpoint {a}/{ep} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_subscription_list(a, ep):
     try:
         endpoint = _get_object_verify_addr(ep, a, endpoint_get)
@@ -807,7 +813,7 @@ def r_subscription_list(a, ep):
         return _response_not_found(f'endpoint {a}/{ep} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_subscription_get(a, ep, s):
     try:
         subscription = _get_object_verify_addr(s,
@@ -822,7 +828,7 @@ def r_subscription_get(a, ep, s):
         return _response_not_found(f'subscription {a}/{ep}/{s} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_subscription_cmd(a, ep, s, cmd):
     try:
         subscription = _get_object_verify_addr(s,
@@ -838,7 +844,7 @@ def r_subscription_cmd(a, ep, s, cmd):
         return _response_not_found(f'subscription {a}/{ep}/{s} not found')
 
 
-@authenticated_method
+@filtered_method
 def r_subscription_create(a, ep, **kwargs):
     addr_id, addr = _process_addr(a)
     try:
@@ -854,7 +860,7 @@ def r_subscription_create(a, ep, **kwargs):
         result['id'], f'addr/{a}/subscription')
 
 
-@authenticated_method
+@filtered_method
 def r_subscription_modify(a, ep, s, **kwargs):
     try:
         subscription = _get_object_verify_addr(s, a, subscription_get)
@@ -876,7 +882,7 @@ def r_subscription_modify(a, ep, s, **kwargs):
         'roboger.subscription') else _response_empty()
 
 
-@authenticated_method
+@filtered_method
 def r_subscription_delete(a, ep, s):
     try:
         subscription = _get_object_verify_addr(s, a, subscription_get)
@@ -902,7 +908,7 @@ _legacy_endpoint_types = {
 _legacy_endpoint_ids = {v: k for k, v in _legacy_endpoint_types.items()}
 
 
-@authenticated_method
+@filtered_method
 def m_addr_list(addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED addr_list')
     if addr_id or addr:
@@ -914,13 +920,13 @@ def m_addr_list(addr_id=None, addr=None, **kwargs):
         return jsonify(addr_list())
 
 
-@authenticated_method
+@filtered_method
 def m_addr_create(**kwargs):
     logger.warning('API DEPRECATED addr_create')
     return jsonify(addr_get(addr_create()))
 
 
-@authenticated_method
+@filtered_method
 def m_addr_change(addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED addr_change')
     try:
@@ -930,7 +936,7 @@ def m_addr_change(addr_id=None, addr=None, **kwargs):
         abort(404)
 
 
-@authenticated_method
+@filtered_method
 def m_addr_set_active(addr_id=None, addr=None, active=1, **kwargs):
     logger.warning('API DEPRECATED addr_set_active')
     try:
@@ -940,7 +946,7 @@ def m_addr_set_active(addr_id=None, addr=None, active=1, **kwargs):
         abort(404)
 
 
-@authenticated_method
+@filtered_method
 def m_addr_enable(addr_id=None, addr=None, active=1, **kwargs):
     logger.warning('API DEPRECATED addr_enable')
     try:
@@ -949,7 +955,7 @@ def m_addr_enable(addr_id=None, addr=None, active=1, **kwargs):
         abort(404)
 
 
-@authenticated_method
+@filtered_method
 def m_addr_disable(addr_id=None, addr=None, active=1, **kwargs):
     logger.warning('API DEPRECATED addr_disable')
     try:
@@ -958,7 +964,7 @@ def m_addr_disable(addr_id=None, addr=None, active=1, **kwargs):
         abort(404)
 
 
-@authenticated_method
+@filtered_method
 def m_addr_delete(addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED addr_delete')
     try:
@@ -968,7 +974,7 @@ def m_addr_delete(addr_id=None, addr=None, **kwargs):
         abort(404)
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_types(**kwargs):
     return jsonify(_legacy_endpoint_types)
 
@@ -1003,7 +1009,7 @@ def _format_legacy_endpoint(endpoint):
     return endpoint
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_list(endpoint_id=None, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED endpoint_list')
     if endpoint_id:
@@ -1075,7 +1081,7 @@ def _format_legacy_endpoint_config(plugin_name, kwargs):
     kwargs['config'] = cfg
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_create(**kwargs):
     logger.warning('API DEPRECATED endpoint_create')
     if 'et' in kwargs:
@@ -1096,7 +1102,7 @@ def m_endpoint_create(**kwargs):
             endpoint_get(endpoint_create(plugin_name, **kwargs))))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_data(endpoint_id,
                     addr_id=None,
                     addr=None,
@@ -1133,7 +1139,7 @@ def m_endpoint_data(endpoint_id,
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_config(endpoint_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED endpoint_config')
     try:
@@ -1156,7 +1162,7 @@ def m_endpoint_config(endpoint_id, addr_id=None, addr=None, **kwargs):
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_skipdups(endpoint_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED endpoint_skipdups [dummy]')
     try:
@@ -1173,7 +1179,7 @@ def m_endpoint_skipdups(endpoint_id, addr_id=None, addr=None, **kwargs):
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_description(endpoint_id,
                            addr_id=None,
                            addr=None,
@@ -1201,7 +1207,7 @@ def m_endpoint_description(endpoint_id,
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_enable(endpoint_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED endpoint_enable')
     try:
@@ -1223,7 +1229,7 @@ def m_endpoint_enable(endpoint_id, addr_id=None, addr=None, **kwargs):
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_disable(endpoint_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED endpoint_disable')
     try:
@@ -1245,7 +1251,7 @@ def m_endpoint_disable(endpoint_id, addr_id=None, addr=None, **kwargs):
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_set_active(endpoint_id,
                           addr_id=None,
                           addr=None,
@@ -1272,7 +1278,7 @@ def m_endpoint_set_active(endpoint_id,
     return jsonify(_format_legacy_endpoint(endpoint))
 
 
-@authenticated_method
+@filtered_method
 def m_endpoint_delete(endpoint_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED endpoint_delete')
     try:
@@ -1309,7 +1315,7 @@ def _format_legacy_subscription(subscription):
     return subscription
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_create(endpoint_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED subscription_create')
     if is_secure_mode():
@@ -1342,7 +1348,7 @@ def m_subscription_create(endpoint_id, addr_id=None, addr=None, **kwargs):
             subscription_get(subscription_create(endpoint_id, **kwargs))))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_list(subscription_id=None,
                         endpoint_id=None,
                         addr_id=None,
@@ -1371,7 +1377,7 @@ def m_subscription_list(subscription_id=None,
         ])
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_delete(subscription_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED subscription_delete')
     if is_secure_mode():
@@ -1389,7 +1395,7 @@ def m_subscription_delete(subscription_id, addr_id=None, addr=None, **kwargs):
     return jsonify(success)
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_enable(subscription_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED subscription_enable')
     if is_secure_mode():
@@ -1408,7 +1414,7 @@ def m_subscription_enable(subscription_id, addr_id=None, addr=None, **kwargs):
     return jsonify(_format_legacy_subscription(subscription))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_disable(subscription_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED subscription_disable')
     if is_secure_mode():
@@ -1427,7 +1433,7 @@ def m_subscription_disable(subscription_id, addr_id=None, addr=None, **kwargs):
     return jsonify(_format_legacy_subscription(subscription))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_set_active(subscription_id,
                               addr_id=None,
                               addr=None,
@@ -1451,7 +1457,7 @@ def m_subscription_set_active(subscription_id,
     return jsonify(_format_legacy_subscription(subscription))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_location(subscription_id,
                             addr_id=None,
                             addr=None,
@@ -1474,7 +1480,7 @@ def m_subscription_location(subscription_id,
     return jsonify(_format_legacy_subscription(subscription))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_keywords(subscription_id,
                             addr_id=None,
                             addr=None,
@@ -1501,7 +1507,7 @@ def m_subscription_keywords(subscription_id,
     return jsonify(_format_legacy_subscription(subscription))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_senders(subscription_id,
                            addr_id=None,
                            addr=None,
@@ -1528,7 +1534,7 @@ def m_subscription_senders(subscription_id,
     return jsonify(_format_legacy_subscription(subscription))
 
 
-@authenticated_method
+@filtered_method
 def m_subscription_level(subscription_id, addr_id=None, addr=None, **kwargs):
     logger.warning('API DEPRECATED subscription_level')
     if is_secure_mode():
